﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public enum TileType
{
    Open,
    Grass,
    Sidewalk,
    House,
    Bush,
}
public class CityGenerator : MonoBehaviour
{
    public GameObject YourHousePrefab = null;
    public GameObject HousePrefab = null;
    public GameObject SidewalkPrefab = null;
    public GameObject BushPrefab = null;

    private TileType[,] _tiles;
    private int mapWidth = 150;
    private int mapHeight = 150;

    public void Awake()
    {
        _tiles = new TileType[mapWidth, mapHeight];
        for (int x = 0; x < mapWidth; x++)
        {
            for (int y = 0; y < mapHeight; y++)
            {
                _tiles[x, y] = TileType.Open;
            }
        }

        GenerateHouses();
        GenerateBushes(0.05f);
    }

    void Start()
    {

    }

    void Update()
    {

    }

    private void GenerateBushes(float bushChance)
    {
        for (int x = 0; x < mapWidth; x++)
        {
            for (int y = 0; y < mapHeight; y++)
            {
                if (_tiles[x, y] == TileType.Open && bushChance > Random.value)
                {
                    PlaceBush(x, y);
                }
            }
        }
    }

    const int houseWidth = 7;
    const int houseHeight = 5;

    private bool NoObstacleForHouseAt(int xPos, int yPos)
    {
        if (xPos <= 0 || yPos <= 0 ||
            xPos >= mapWidth - houseWidth - 1 ||
            yPos >= mapHeight - houseHeight - 1)
        {
            return false;
        }

        for (int x = xPos; x < xPos + houseWidth; x++)
        {
            for (int y = yPos; y < yPos + houseHeight + 1; y++)
            {
                if (x >= mapWidth || y >= mapHeight)
                {
                    print("Error");
                }
                if (_tiles[x, y] != TileType.Open)
                {
                    return false;
                }
            }
        }

        return true;
    }

    private Vector3 TileCoordToWorldpoint(int x, int y)
    {
        Vector3 result = new Vector3(x - (mapWidth / 2), y - (mapHeight / 2), 0.0f);

        return result;
    }

    private void SpawnHouseAt(GameObject prefab, int xPos, int yPos)
    {
        Vector3 spawnLoc = TileCoordToWorldpoint(xPos + ((houseWidth) / 2), yPos + ((houseHeight) / 2) + 1);

        GameObject house = GameObject.Instantiate(prefab, spawnLoc, Quaternion.identity) as GameObject;
        house.transform.SetParent(gameObject.transform);

        // Fill the area in on the tile map
        for (int x = xPos; x < xPos + houseWidth; x++)
        {

            for (int y = yPos + 1; y < yPos + houseHeight + 1; y++)
            {
                _tiles[x, y] = TileType.House;
            }

        }

        // Drop a little sidewalk
        PlaceSidewalk(xPos + (houseWidth / 2), yPos);

    }

    private bool AttemptToPlaceHouse(GameObject prefab, int xHousePos, int yHousePos)
    {
        if (NoObstacleForHouseAt(xHousePos, yHousePos))
        {
            SpawnHouseAt(prefab, xHousePos, yHousePos);
            return true;
        }
        return false;
    }

    private void GenerateHouse(GameObject prefab)
    {
        int xHousePos = Random.Range(1, mapWidth - houseWidth);
        int yHousePos = Random.Range(1, mapHeight - houseHeight);

        AttemptToPlaceHouse(prefab, xHousePos, yHousePos);
    }

    private void PlaceBush(int x, int y)
    {
        if (x > 0 && x < mapWidth && y > 0 && y < mapHeight)
        {
            _tiles[x, y] = TileType.Bush;
            GameObject sidewalk = GameObject.Instantiate(BushPrefab, TileCoordToWorldpoint(x, y), Quaternion.identity) as GameObject;
            sidewalk.transform.SetParent(gameObject.transform);

        }
    }

    private void PlaceSidewalk(int x, int y)
    {
        if (x > 0 && x < mapWidth && y > 0 && y < mapHeight)
        {
            _tiles[x, y] = TileType.Sidewalk;
            GameObject sidewalk = GameObject.Instantiate(SidewalkPrefab, TileCoordToWorldpoint(x, y), Quaternion.identity) as GameObject;
            sidewalk.transform.SetParent(gameObject.transform);

        }
    }

    private void MakeVerticalSidewalkFrom(int xPos, int yPos)
    {
        int nSafetyCheck = 100;  // Since we are doing a potential infinite loop, this is a safety count

        // Locate the first non-sidewalk tile walking downwards
        while (_tiles[xPos, yPos] == TileType.Sidewalk && nSafetyCheck > 0)
        {
            yPos--;
            nSafetyCheck--;
        }

        // Draw sidewalk down to the end
        while (yPos > 0 && yPos < mapHeight && _tiles[xPos, yPos] == TileType.Open && nSafetyCheck > 0)
        {
            PlaceSidewalk(xPos, yPos);
            yPos--;
            nSafetyCheck--;

        }

    }

    List<int> lastRowPositions = new List<int>();
    List<int> thisRowPositions = new List<int>();

    private void GenerateRowOfHouses(int yBaseline, bool spawnYourHouse)
    {
        // Draw a sidewalk that is mostly horizontal from west to east that jitters occasionally
        int yPosition = yBaseline;
        int xStreak = 0;

        int[] highestYs = new int[mapWidth];

        // Generate the sidewalk
        for (int xPosition = 0; xPosition < mapWidth; xPosition++)
        {
            xStreak--;
            int previousY = yPosition; // This is used to keep track of the highest sidewalk generated this turn so we know what y height to try and spawn a house

            // Spawn the sidewalk
            PlaceSidewalk(xPosition, yPosition);

            // If time to jitter, jitter and place an extra sidewalk
            if (xStreak <= 0)
            {
                xStreak = Random.Range(5, 5 + houseWidth);

                if (yPosition != yBaseline)
                {
                    yPosition = yBaseline;
                }
                else
                {
                    // Jitter the sidewalk up or down vertically 1
                    yPosition += Random.Range(-1, 1);
                }

                // Place an extra sidewalk if we modified our Y
                if (yPosition != previousY)
                    PlaceSidewalk(xPosition, yPosition);


            }

            highestYs[xPosition] = ((previousY > yPosition) ? previousY : yPosition);

        }

        // Places houses all along the sidewalk
        int xMidPoint = mapWidth / 2;
        for (int xPosition = houseWidth / 2; xPosition < mapWidth; xPosition++)
        {
            int xCornerOfHouse = xPosition - (houseWidth / 2);
            bool bPlaced = false;
            if (spawnYourHouse && xPosition > xMidPoint)
            {
                bPlaced = AttemptToPlaceHouse(YourHousePrefab, xCornerOfHouse, highestYs[xPosition] + 1);

                if (bPlaced)
                {
                    spawnYourHouse = false;

                    // Reposition the player
                    GameObject playerObj = GameObject.FindGameObjectWithTag("Player");
                    playerObj.transform.position = TileCoordToWorldpoint(xPosition, highestYs[xPosition]);
                }

            }
            else
            {
                bPlaced = AttemptToPlaceHouse(HousePrefab, xCornerOfHouse, highestYs[xPosition] + 1);

            }

            if (bPlaced)
            {
                xPosition += houseWidth + Random.Range(1, 3);
                thisRowPositions.Add(xCornerOfHouse);
            }
        }

        // Do a pass over the previous row corners and decide where to make a vertical sidewalk
        for (int xVertSidewalkEntry = 1; xVertSidewalkEntry < lastRowPositions.Count; xVertSidewalkEntry++)
        {
            if (lastRowPositions[xVertSidewalkEntry] - lastRowPositions[xVertSidewalkEntry - 1] == houseWidth + 3)
            {
                int xMakeSidewalkAt = lastRowPositions[xVertSidewalkEntry] - 2;
                MakeVerticalSidewalkFrom(xMakeSidewalkAt, highestYs[xMakeSidewalkAt]);
            }
        }

        // Swap the new row positions for the old row positions
        lastRowPositions = thisRowPositions;
        thisRowPositions = new List<int>();

    }

    public void GenerateHouses()
    {
        int midPoint = mapHeight / 2;
        bool spawnedYourHouse = false;

        for (int y = 2; y < mapHeight; y += houseHeight + 5)
        {
            if (!spawnedYourHouse && y > midPoint)
            {
                GenerateRowOfHouses(y, true);
                spawnedYourHouse = true;
            }
            else
            {
                GenerateRowOfHouses(y, false);
            }
        }

    }
}
