﻿using UnityEngine;
using UnityEngine.UI;
using System.Collections;


public enum HouseState
{
    WaitingForKnock,
    WaitingToOpenDoor,
    WaitingToGiveCandy,
    GaveCandy
}

public static class ColorHelper
{
    public static Color HSVToRGB(float H, float S, float V)
    {
        if (S == 0f)
            return new Color(V, V, V);
        else if (V == 0f)
            return Color.black;
        else
        {
            Color col = Color.black;
            float Hval = H * 6f;
            int sel = Mathf.FloorToInt(Hval);
            float mod = Hval - sel;
            float v1 = V * (1f - S);
            float v2 = V * (1f - S * mod);
            float v3 = V * (1f - S * (1f - mod));
            switch (sel + 1)
            {
                case 0:
                    col.r = V;
                    col.g = v1;
                    col.b = v2;
                    break;
                case 1:
                    col.r = V;
                    col.g = v3;
                    col.b = v1;
                    break;
                case 2:
                    col.r = v2;
                    col.g = V;
                    col.b = v1;
                    break;
                case 3:
                    col.r = v1;
                    col.g = V;
                    col.b = v3;
                    break;
                case 4:
                    col.r = v1;
                    col.g = v2;
                    col.b = V;
                    break;
                case 5:
                    col.r = v3;
                    col.g = v1;
                    col.b = V;
                    break;
                case 6:
                    col.r = V;
                    col.g = v1;
                    col.b = v2;
                    break;
                case 7:
                    col.r = V;
                    col.g = v3;
                    col.b = v1;
                    break;
            }
            col.r = Mathf.Clamp(col.r, 0f, 1f);
            col.g = Mathf.Clamp(col.g, 0f, 1f);
            col.b = Mathf.Clamp(col.b, 0f, 1f);
            return col;
        }
    }
}

public class House : MonoBehaviour
{

    private HouseState state = HouseState.WaitingForKnock;
    private float timeToNextState = 0.0f;       // Time that we are going to move to the next game state
    private float candyHandoutGiveupTime = 0.0f; // Time after opening the door before we give up trying to give candy
    private int numberOfExcessivePlayerDoorknocks = 0;

    private GameObject door = null;
    private GameObject doorknockPlayer = null;

    public GameObject pumpkinPrefab = null;

    public GameObject knockFlashPrefab = null;

    private float timeToKnockAllowed = 0.0f;
    private int numPumpkins = 0;

    public bool IsTutorial = false;
    public int TutorialNumPumpkins = 0;
    public int TutorialNumLitWindows = 0;
    public float TutorialGuaranteedRoll = 0.0f;
    
    // Use this for initialization
    void Start()
    {
        EventManager.instance.AddListener<DoorknockEvent>(OnDoorknock);

        door = transform.Find("HouseDoor").gameObject;

        // Assign myself a random color
        SpriteRenderer sr = GetComponent<SpriteRenderer>();
        if (sr != null)
        {
            sr.color = ColorHelper.HSVToRGB(Random.value, 0.4f, 1.0f);
        }

        // Randomize the position of the chimney
        Transform chimneyTransform = gameObject.transform.Find("Chimney");
        if (chimneyTransform != null)
        {
            chimneyTransform.localPosition = new Vector3(
                chimneyTransform.localPosition.x + Random.Range(-2.15f, 2.15f),
                Random.Range(0.85f, 1.92f),
                chimneyTransform.localPosition.z
                );

        }

        if (IsTutorial)
        {
            numPumpkins = TutorialNumPumpkins;

            // Iterate the lights turning them on or off
            gameObject.transform.Find("Window1").gameObject.GetComponent<Window>().isLit = false;
            gameObject.transform.Find("Window2").gameObject.GetComponent<Window>().isLit = false;
            gameObject.transform.Find("Window3").gameObject.GetComponent<Window>().isLit = false;
            gameObject.transform.Find("Window4").gameObject.GetComponent<Window>().isLit = false;

            if (TutorialNumLitWindows > 0)
            {
                gameObject.transform.Find("Window3").gameObject.GetComponent<Window>().isLit = true;
                if (TutorialNumLitWindows > 1)
                {
                    gameObject.transform.Find("Window1").gameObject.GetComponent<Window>().isLit = true;

                    if (TutorialNumLitWindows > 2)
                    {
                        gameObject.transform.Find("Window4").gameObject.GetComponent<Window>().isLit = true;
                        if (TutorialNumLitWindows > 3)
                        {
                            gameObject.transform.Find("Window2").gameObject.GetComponent<Window>().isLit = true;
                        }
                    }
                }
            }
        }
        else
        {

            // Place pumpkins in front of every house.
            float roll = Random.value;
            if (roll < 0.7f)
            {
                numPumpkins = Random.Range(0, 5);
            }
            else if (roll < 0.9f)
            {
                numPumpkins = Random.Range(5, 7);
            }
            else
            {
                // JACKPOT HOUSE
                numPumpkins = Random.Range(12, 15);
            }
        }

        AddPumpkins(numPumpkins);
    }

    public void OnDestroy()
    {
        EventManager.instance.RemoveListener<DoorknockEvent>(OnDoorknock);

    }

    private void AddSinglePumpkin()
    {
        if (pumpkinPrefab != null)
        {
            float xPos = Random.Range(0.8f, 3.2f);
            if (Random.value < 0.5f)
            {
                xPos = -xPos;
            }

            Vector3 spawnPosition = new Vector3(xPos, -2.6f, 0.0f);
            GameObject pumpkin = GameObject.Instantiate(pumpkinPrefab, spawnPosition, Quaternion.identity) as GameObject;
            pumpkin.transform.SetParent(gameObject.transform, false);

            pumpkin.transform.localScale = pumpkin.transform.localScale * Random.Range(0.8f, 1.2f);
        }

    }

    public void AddPumpkins(int n)
    {
        for (int i = 0; i < n; i++)
        {
            AddSinglePumpkin();
        }

    }

    int DetermineHowMuchCandyToGive()
    {
        int result = 3;

        if (numberOfExcessivePlayerDoorknocks > 0)
        {
            result = 1;
        }
        else
        {
            result += numPumpkins;

        }

        return result;
    }

    int CheckForCuteness(int originalAmount)
    {
        int result = originalAmount;
        if (numberOfExcessivePlayerDoorknocks == 0)
        {

            if (PlayerStats.GetInstance().HasUpgrade(CandyUpgrade.BonusCandyFromHouses))
            {
                result += 2;
                GameObject textObj = gameObject.FloatText("What a cute monster!\n+2 Candy");
                textObj.transform.position += new Vector3(-3.0f, -2f, 0f);
                textObj.transform.localRotation = Quaternion.Euler(0f, 0f, 20f);

                Text t = textObj.GetComponentInChildren<Text>();
                t.fontSize /= 2;
            }
        }

        return result;
    }

    private string GetRandomHostileGreeting()
    {
        if (Random.value < 0.3f)
            return "Man you knock a lot";

        return "Quit knocking so much";
    }

    private string GetRandomFriendlyGreeting()
    {
        if (Random.value < 0.3f)
            return "Trick or Treat!";

        return "Hello";
    }

    private void OpenDoor()
    {
        // Open door here
        door.GetComponent<Door>().SetDoorOpen(true);
    }
    private void CloseDoor()
    {
        // Open door here
        door.GetComponent<Door>().SetDoorOpen(false);
    }


    void IssueGreeting()
    {
        OpenDoor();

        // And say hello
        if (numberOfExcessivePlayerDoorknocks > 0)
        {
            gameObject.FloatText(GetRandomHostileGreeting());
        }
        else
        {
            gameObject.FloatText(GetRandomFriendlyGreeting());
        }
    }

    bool PlayerIsInRangeOfDoor()
    {
        float distance = Vector3.Distance(doorknockPlayer.transform.position, door.transform.position);

        return distance < 1.5f;
    }

    //  Update is called once per frame
    void Update()
    {

        if (state == HouseState.WaitingToOpenDoor && Time.time >= timeToNextState)
        {
            // It's candy time!!!
            IssueGreeting();

            state++;
            timeToNextState = Time.time + 1.0f;
            candyHandoutGiveupTime = Time.time + 4.0f;
        }
        else if (state == HouseState.WaitingToGiveCandy && Time.time > timeToNextState)
        {
            if (PlayerIsInRangeOfDoor())
            {
                GiveTheCandy();
                state++;
                CloseDoor();
            }
            else if (Time.time > candyHandoutGiveupTime)
            {
                GiveUpOnGivingCandy();
                state = HouseState.WaitingForKnock;
                CloseDoor();

            }
        }
    }


    bool GiveTheCandy()
    {
        bool result = false;

        PlayerStats ps = PlayerStats.GetInstance();

        if (ps.Candy >= ps.CandyCapacity)
        {
            // Pumpkin is totally full
            GameObject floatText = gameObject.FloatText("Your pumpkin is full!\nGo home and stash it");
            floatText.GetComponentInChildren<Text>().color = new Color(0.8f, 0.1f, 0.1f);
        }
        else
        {
            // Pumpkin has room
            int candyToGive = DetermineHowMuchCandyToGive();

            if (ps.Candy + candyToGive > ps.CandyCapacity)
            {
                // Pumpkin is partially full
                candyToGive = ps.CandyCapacity - ps.Candy;
                gameObject.FloatText("Looks like " + candyToGive + " fills you up!");
            }
            else
            {
                // Pumpkin has room!
                gameObject.FloatText("Have " + candyToGive + " candy");
            }

            candyToGive = CheckForCuteness(candyToGive);

            PlayerStats.GetInstance().GiveCandy(candyToGive);
        }

        TriggerDoorknockCooldown(1.0f);

        return result;
    }

    void GiveUpOnGivingCandy()
    {
        gameObject.FloatText("I guess nobody was there");
        if (GetNumberOfLightsOn() < 4)
        { 
            TurnOnALight();
        }

        TriggerDoorknockCooldown(1.0f);
    }

    void TriggerDoorknockCooldown(float duration)
    {
        timeToKnockAllowed = Time.time + duration;
    }

    void OnDoorknock(DoorknockEvent e)
    {
        if (this != null && e.houseGameObject == gameObject)
        {
            if (Time.time > timeToKnockAllowed)
            {
                doorknockPlayer = e.playerGameObject;

                switch (state)
                {
                    case HouseState.WaitingForKnock:
                        HandleKnock_WaitingForKnock();
                        break;
                    case HouseState.WaitingToOpenDoor:
                        HandleKnock_WaitingToOpenDoor();
                        break;
                    case HouseState.WaitingToGiveCandy:
                        HandleKnock_WaitingToGiveCandy();
                        break;
                    case HouseState.GaveCandy:
                        HandleKnock_GaveCandy();
                        break;
                }
            }

        }
    }

    private int GetNumberOfLightsOn()
    {
        int result = 0;

        if (gameObject.transform.Find("Window1").gameObject.GetComponent<Window>().isLit)
            result++;

        if (gameObject.transform.Find("Window2").gameObject.GetComponent<Window>().isLit)
            result++;

        if (gameObject.transform.Find("Window3").gameObject.GetComponent<Window>().isLit)
            result++;

        if (gameObject.transform.Find("Window4").gameObject.GetComponent<Window>().isLit)
            result++;

        return result;
    }

    private float ChanceToGiveCandy()
    {
        float flChance = 0.25f * GetNumberOfLightsOn();

        HUDManager.GetInstance().SetDebugString("Chance for Candy: " + flChance * 100.0f + "%");

        return flChance;
    }

    private bool RollToGiveOutCandy()
    {
        float roll = Random.value;
        if (IsTutorial)
            roll = TutorialGuaranteedRoll;

        return roll < ChanceToGiveCandy();
    }

    private void DetermineTimeToGiveCandy()
    {
        timeToNextState = Time.time + Random.Range(1.0f, GetMaxHomeownerResponseTime());
    }

    private void KnockOnDoor()
    {
        if (door != null)
        {
            gameObject.GetComponent<AudioSource>().Play();

            // Create two Knock Flashes
            GameObject.Instantiate(knockFlashPrefab, door.transform.position + new Vector3(Random.Range(-0.2f, 0.2f), Random.Range(0.2f, 0.6f)), Quaternion.identity);

        }
    }

    private bool lightStateChangeQueued = false;
    public GameObject firecrackerPrefab = null;
    void HandleKnock_WaitingForKnock()
    {
        KnockOnDoor();
        
        if (GetNumberOfLightsOn() == 0 && PlayerStats.GetInstance().HasUpgrade(CandyUpgrade.Firecrackers))
        {
            PlayerStats.GetInstance().GiveCandy(15);
            GameObject obj = GameObject.Instantiate(firecrackerPrefab, door.transform.position, Quaternion.identity) as GameObject;
            obj.FloatText("Firecrackers!\n15 candy!");

            TurnOnALight();
            TurnOnALight();
            TurnOnALight();
            TurnOnALight();

            state = HouseState.GaveCandy;
        }
        else
        {
            bool giveOutCandy = RollToGiveOutCandy();

            if (giveOutCandy)
            {
                state++;
                DetermineTimeToGiveCandy();
            }
            else
            {
                // Queue a light state change
                if (!lightStateChangeQueued)
                {
                    lightStateChangeQueued = true;
                    Invoke("TurnOnALight", GetMaxHomeownerResponseTime());
                }
            }
        }
    }

    private float GetMaxHomeownerResponseTime()
    {
        if (GetNumberOfLightsOn() == 4)
        {
            return 1.0f;
        }

        const float MaxTimeWithNoLights = 6.0f;
        float flRatio = (4f - GetNumberOfLightsOn()) / 4.0f;

        return (MaxTimeWithNoLights * flRatio) + 2.0f;
    }

    void TurnOnALight()
    {
        if (GetNumberOfLightsOn() > 3)
        {
            return; // Can't turn on lights when there's already 4 on
        }

        lightStateChangeQueued = false;

        // Turn on one of the lights
        bool bFound = false;
        int nLightIndexToPick = Random.Range(1, 5);

        Window w = null;

        // Iterate the lights sequentially until we find one
        while (!bFound)
        {
            string windowName = "Window" + nLightIndexToPick;
            w = gameObject.transform.Find(windowName).gameObject.GetComponent<Window>();

            if (!w.isLit)
                bFound = true;
            else
            {
                nLightIndexToPick += 1;
                if (nLightIndexToPick > 4)  // Wrap around back to 1
                    nLightIndexToPick = 1;
            }

        }

        if (w != null)
            w.isLit = true;
    }

    void HandleKnock_WaitingToOpenDoor()
    {
        KnockOnDoor();
        numberOfExcessivePlayerDoorknocks++;
    }

    void HandleKnock_WaitingToGiveCandy()
    {
        // "Knocking" when the door is already open does not do anything
        ;

    }

    void HandleKnock_GaveCandy()
    {
        KnockOnDoor();
        OpenDoor();

        Invoke("GoAwayKid", 1.0f);
    }

    void GoAwayKid()
    {
        CloseDoor();

        gameObject.FloatText("I already gave you candy!");
    }


}

